# Copyright (C) 2019-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os
import subprocess
from enum import Enum
from os import PathLike
from pathlib import Path
from tempfile import NamedTemporaryFile
from types import TracebackType
from typing import Any, Iterable, Iterator, List, Optional, Type

from autohooks.utils import GitError, exec_git, get_project_root_path

__all__ = [
    "exec_git",
    "get_staged_status",
    "get_status",
    "is_partially_staged_status",
    "is_staged_status",
    "stage_files",
    "stash_unstaged_changes",
    "Status",
    "StatusEntry",
]


def _get_git_toplevel_path():
    try:
        git_dir = exec_git("rev-parse", "--show-toplevel").rstrip()
    except subprocess.CalledProcessError as e:
        print("could not determine toplevel directory. {e.output.decode()}")
        raise e from None
    return Path(git_dir).resolve()


class Status(Enum):
    UNMODIFIED = " "
    MODIFIED = "M"
    ADDED = "A"
    DELETED = "D"
    RENAMED = "R"
    COPIED = "C"
    UPDATED = "U"
    UNTRACKED = "?"
    IGNORED = "!"


class StatusEntry:
    def __init__(self, status_string: str, root_path: Path = None) -> None:
        status = status_string[:2]
        filename = status_string[3:]

        self.index = Status(status[0])
        self.working_tree = Status(status[1])
        self.root_path = root_path

        if self.index == Status.RENAMED:
            new_filename, old_filename = filename.split("\0")
            self.path = Path(new_filename)
            self.old_path = Path(old_filename)
        else:
            self.path = Path(filename)

    def __str__(self) -> str:
        return f"{self.index.value}{self.working_tree.value} {str(self.path)}"

    def __repr__(self) -> str:
        return f"<StatusEntry {str(self)}>"

    def absolute_path(self) -> Path:
        if self.root_path:
            return (self.root_path / self.path).resolve()
        return self.path.resolve()

    def __fspath__(self):
        return self.path.__fspath__()


def _parse_status(output: str) -> Iterator[str]:
    output = output.rstrip("\0")
    if not output:
        return

    output_list = output.split("\0")
    while output_list:
        line = output_list.pop(0)
        if line[0] == Status.RENAMED.value:
            yield f"{line}\0{output_list.pop(0)}"
        else:
            yield line


def is_staged_status(status: StatusEntry) -> bool:
    """Returns true, if the status of the given StatusEntry is staged

    Arguments:
        status: A StatusEntry object that contains the filename,
                path and the git status

    Returns:
        True if file is staged, False else
    """
    return (
        status.index != Status.UNMODIFIED
        and status.index != Status.UNTRACKED
        and status.index != Status.IGNORED
        and status.index != Status.DELETED
    )


def is_partially_staged_status(status: StatusEntry) -> bool:
    """Returns true, if the status of the given StatusEntry
    is partially staged

    Arguments:
        status: A StatusEntry object that contains the filename,
                path and the git status

    Returns:
        True if file is partially staged, False else
    """
    return (
        status.index != Status.UNMODIFIED
        and status.index != Status.UNTRACKED
        and status.index != Status.IGNORED
        and status.index != Status.DELETED
        and status.working_tree != Status.UNMODIFIED
        and status.working_tree != Status.UNTRACKED
        and status.working_tree != Status.IGNORED
    )


def get_status(files: Optional[Iterable[PathLike]] = None) -> List[StatusEntry]:
    """Get information about the current git status

    Arguments:
        files: (optional) specify a list of files and exclude all other pathes
               for the status

    Returns:
        A list of StatusEntries that contains the status of the specific files
    """
    args = [
        "status",
        "-z",
        "--ignore-submodules",
        "--untracked-files=no",
    ]

    if files is not None:
        args.append("--")
        args.extend([os.fspath(f) for f in files])

    output = exec_git(*args)
    root_path = _get_git_toplevel_path()
    return [StatusEntry(f, root_path) for f in _parse_status(output)]


def get_staged_status(
    files: Optional[Iterable[PathLike]] = None,
) -> List[StatusEntry]:
    """Get a list of StatusEntries containing only staged files

    Arguments:
        files: (optional) specify a list of files and exclude all other pathes
               for the status

    Returns:
        A list of StatusEntries with files that are staged
    """
    status = get_status(files)
    return [s for s in status if is_staged_status(s)]


def stage_files_from_status_list(status_list: Iterable[StatusEntry]) -> None:
    """Add the passed files from the status list to git staging index

    Deprecated. Please use stage_files instead.

    Arguments:
        status_list: A List of StatusEntries that should be added
    """
    stage_files(status_list)


def stage_files(files: Iterable[PathLike]) -> None:
    """Add the passed files to git staging index

    Arguments:
        files: A List of files to add to the index
    """
    filenames = [os.fspath(f) for f in files]
    exec_git("add", *filenames)


def get_diff(files: Optional[Iterable[StatusEntry]] = None) -> str:
    """Get the diff of the passed files

    Arguments:
        status_list: A List of StatusEntries that should be diffed

    Returns:
        string containing the diff of the given files
    """
    args = ["--no-pager", "diff"]

    if files is not None:
        args.append("--")
        args.extend([str(f.absolute_path()) for f in files])

    return exec_git(*args)


def _write_tree() -> str:
    """
    Create a tree object from the current index

    Returns:
        The name of the new tree object
    """
    return exec_git("write-tree").strip()


def _read_tree(ref_or_hashid: str) -> None:
    """
    Loads (reads) tree information into the index

    Arguments:
        ref_or_hashid: Git hash or ref to load into the index
    """
    exec_git("read-tree", ref_or_hashid)


def _checkout_from_index(files: Iterable[PathLike]) -> None:
    """
    Copy all files listed from the index to the working directory

    Arguments:
        files: Iterable of files that should be checked out into the working
               directory
    """
    filenames = [os.fspath(s) for s in files]
    exec_git("checkout-index", "-f", "--", *filenames)


def _set_ref(name: str, hashid: str) -> None:
    """
    Create a git ref for a hash

    Arguments:
        name: Name of the reference to be created
        hashid: Hash that the reference should point to
    """
    exec_git("update-ref", name, hashid)


def _get_tree_diff(tree1: str, tree2: str) -> bytes:
    """
    Calculate a diff between two tree objects

    Returns:
        The diff as bytes
    """
    return subprocess.check_output(
        [
            "git",
            "diff-tree",
            "--ignore-submodules",
            "--binary",
            "--no-color",
            "--no-ext-diff",
            "--unified=0",
            tree1,
            tree2,
        ]
    )


def _apply_diff(patch: bytes) -> None:
    with NamedTemporaryFile(mode="wb", buffering=0) as f:
        f.write(patch)

        exec_git(
            "apply",
            "-v",
            "--whitespace=nowarn",
            "--reject",
            "--recount",
            "--unidiff-zero",
            f.name,
        )


INDEX_REF = "refs/autohooks/index"
WORKING_REF = "refs/autohooks/working"


class stash_unstaged_changes:  # pylint: disable=invalid-name
    """
    A context manager that stashes changes on tracked files that are not added
    to the index. The stashed changes are restored when the context manager
    exits.

    with stash_unstaged_changes():
        do_something()
    """

    def __init__(self, files: Optional[Iterable[PathLike]] = None) -> None:
        status_list = get_status(files)
        self.partially_staged = [
            s for s in status_list if is_partially_staged_status(s)
        ]

    def stash_changes(self) -> None:
        # save current staging area aka. index
        self.index = _write_tree()
        # add ref to be able to restore index manually
        _set_ref(INDEX_REF, self.index)
        # add changes from files to index
        stage_files(self.partially_staged)
        # save index as working tree
        # unstaged changes are stored in the working tree now
        self.working_tree = _write_tree()
        # add ref to be able to restore working tree manually
        _set_ref(WORKING_REF, self.working_tree)

        # restore index without working tree changes
        # working tree changes are "stashed" now
        _read_tree(self.index)
        _checkout_from_index(self.partially_staged)

    def restore_working_tree(self) -> None:
        # restore working tree
        _read_tree(self.working_tree)
        # checkout working tree
        _checkout_from_index(self.partially_staged)

    def __enter__(self) -> None:
        if self.partially_staged:
            self.stash_changes()

    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> Any:
        if not self.partially_staged:
            return

        if exc_type is not None:
            # an error has occurred
            # restore working tree and index as it was before formatting
            self.restore_working_tree()
            _read_tree(self.index)
        else:
            # save possible changes made to the index
            changed_tree = _write_tree()

            self.restore_working_tree()

            # restore index
            _read_tree(changed_tree)

            # create and apply diff between index before running the plugin and
            # changes made by the plugin if some changes have been applied and
            # staged.
            # changed_tree will be the same as index if no changes are applied.
            # changed_tree may be the same as the working tree. in that case no
            # further action is needed.
            if changed_tree != self.index and changed_tree != self.working_tree:
                # create diff between working tree and changed tree
                patch = _get_tree_diff(self.index, changed_tree)
                try:
                    # apply diff to working tree
                    _apply_diff(patch)
                except GitError as e:
                    print(
                        "Found conflicts between plugin and local changes. "
                        "Plugin changes will be ignored for conflicted hunks.",
                        e,
                    )

                    rootpath = get_project_root_path()
                    for path in rootpath.glob("*.rej"):
                        path.unlink()



# -*- coding: utf-8 -*-
# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import tempfile
from contextlib import contextmanager
from pathlib import Path
from typing import Generator

from autohooks.utils import exec_git


@contextmanager
def tempdir(change_into=False) -> Generator[Path, None, None]:
    temp_dir = tempfile.TemporaryDirectory()

    if change_into:
        os.chdir(temp_dir.name)

    yield Path(temp_dir.name)

    temp_dir.cleanup()


@contextmanager
def tempgitdir() -> Generator[Path, None, None]:
    temp_dir = tempfile.TemporaryDirectory()
    temp_path = Path(temp_dir.name)

    os.chdir(str(temp_path))

    exec_git("init", "-b", "main")
    exec_git("config", "--local", "user.email", "max.mustermann@example.com")
    exec_git("config", "--local", "user.name", "Max Mustermann")

    yield temp_path

    temp_dir.cleanup()



# Copyright (C) 2019-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from typing import Optional

from pontos.terminal.rich import RichTerminal as Terminal
from pontos.terminal.terminal import Signs
from rich.progress import BarColumn
from rich.progress import Progress as RichProgress
from rich.progress import SpinnerColumn, TaskProgressColumn, TextColumn

__all__ = (
    "Terminal",
    "Progress",
    "Signs",
    "bold_info",
    "error",
    "fail",
    "info",
    "ok",
    "out",
    "warning",
)

__term = None  # pylint: disable=invalid-name


def ok(message: str) -> None:
    __term.ok(message)


def fail(message: str) -> None:
    __term.fail(message)


def error(message: str) -> None:
    __term.error(message)


def warning(message: str) -> None:
    __term.warning(message)


def info(message: str) -> None:
    __term.info(message)


def bold_info(message: str) -> None:
    __term.bold_info(message)


def out(message: str):
    __term.print(message)


def overwrite(message: str, new_line: bool = False):
    __term.print_overwrite(message, new_line=new_line)


def _set_terminal(term: Optional[Terminal] = None) -> Terminal:
    global __term  # pylint: disable=global-statement, invalid-name
    if not term:
        __term = Terminal()
    else:
        __term = term
    return __term


class Progress(RichProgress):
    def __init__(self, terminal: Terminal) -> None:
        super().__init__(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            BarColumn(),
            TaskProgressColumn(),
            console=terminal._console,
            transient=True,
        )

    def finish_task(self, task_id):
        self.update(task_id, total=1, advance=1)



# Copyright (C) 2019-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from enum import Enum


class Mode(Enum):
    PIPENV = 1
    PYTHONPATH = 2
    POETRY = 3
    PIPENV_MULTILINE = 4
    POETRY_MULTILINE = 5
    UNDEFINED = -1
    UNKNOWN = -2

    def get_effective_mode(self):
        # pylint: disable=comparison-with-callable
        if self.value == Mode.PIPENV.value:
            return Mode.PIPENV
        if self.value == Mode.PIPENV_MULTILINE.value:
            return Mode.PIPENV_MULTILINE
        if self.value == Mode.POETRY.value:
            return Mode.POETRY
        if self.value == Mode.POETRY_MULTILINE.value:
            return Mode.POETRY_MULTILINE
        return Mode.PYTHONPATH

    @staticmethod
    def from_string(modestring: str) -> "Mode":
        if not modestring:
            return Mode.UNDEFINED

        try:
            return Mode[modestring.upper()]
        except KeyError:
            return Mode.UNKNOWN

    def __str__(self) -> str:
        return self.name.lower()  # pylint: disable=no-member



# Copyright (C) 2019-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

from autohooks.settings import Mode


class ModeTestCase(unittest.TestCase):
    def test_get_effective_mode(self):
        self.assertEqual(Mode.PIPENV.get_effective_mode(), Mode.PIPENV)
        self.assertEqual(Mode.PYTHONPATH.get_effective_mode(), Mode.PYTHONPATH)
        self.assertEqual(
            Mode.PIPENV_MULTILINE.get_effective_mode(), Mode.PIPENV_MULTILINE
        )
        self.assertEqual(Mode.POETRY.get_effective_mode(), Mode.POETRY)
        self.assertEqual(
            Mode.POETRY_MULTILINE.get_effective_mode(), Mode.POETRY_MULTILINE
        )
        self.assertEqual(Mode.UNDEFINED.get_effective_mode(), Mode.PYTHONPATH)
        self.assertEqual(Mode.UNKNOWN.get_effective_mode(), Mode.PYTHONPATH)

    def test_get_pipenv_mode_from_string(self):
        self.assertEqual(Mode.from_string("pipenv"), Mode.PIPENV)
        self.assertEqual(Mode.from_string("PIPENV"), Mode.PIPENV)

    def test_get_pipenv_multiline_mode_from_string(self):
        self.assertEqual(
            Mode.from_string("pipenv_multiline"), Mode.PIPENV_MULTILINE
        )
        self.assertEqual(
            Mode.from_string("PIPENV_MULTILINE"), Mode.PIPENV_MULTILINE
        )

    def test_get_pythonpath_mode_from_string(self):
        self.assertEqual(Mode.from_string("pythonpath"), Mode.PYTHONPATH)
        self.assertEqual(Mode.from_string("PYTHONPATH"), Mode.PYTHONPATH)

    def test_get_poetry_mode_from_string(self):
        self.assertEqual(Mode.from_string("poetry"), Mode.POETRY)
        self.assertEqual(Mode.from_string("POETRY"), Mode.POETRY)

    def test_get_poetry_multiline_mode_from_string(self):
        self.assertEqual(
            Mode.from_string("poetry_multiline"), Mode.POETRY_MULTILINE
        )
        self.assertEqual(
            Mode.from_string("POETRY_MULTILINE"), Mode.POETRY_MULTILINE
        )

    def test_get_invalid_mode_from_string(self):
        self.assertEqual(Mode.from_string("foo"), Mode.UNKNOWN)
        self.assertEqual(Mode.from_string(None), Mode.UNDEFINED)
        self.assertEqual(Mode.from_string(""), Mode.UNDEFINED)

    def test_string_conversion(self):
        self.assertEqual(str(Mode.PIPENV), "pipenv")
        self.assertEqual(str(Mode.PIPENV_MULTILINE), "pipenv_multiline")
        self.assertEqual(str(Mode.PYTHONPATH), "pythonpath")
        self.assertEqual(str(Mode.POETRY_MULTILINE), "poetry_multiline")
        self.assertEqual(str(Mode.POETRY), "poetry")
        self.assertEqual(str(Mode.UNKNOWN), "unknown")
        self.assertEqual(str(Mode.UNDEFINED), "undefined")


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2019-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path
from typing import Any, Dict, List, Union

import tomlkit

from autohooks.settings import Mode
from autohooks.utils import get_pyproject_toml_path, is_split_env

AUTOHOOKS_SECTION = "tool.autohooks"


class Config:
    def __init__(self, config_dict: Dict = None) -> None:
        self._config_dict = config_dict or {}

    def get(self, *keys: str) -> "Config":
        config_dict = self._config_dict

        for key in keys:
            config_dict = config_dict.get(key, {}).copy()

        return Config(config_dict)

    def get_value(self, key: str, default: Any = None) -> Union[str, List[str]]:
        return self._config_dict.get(key, default)

    def is_empty(self) -> bool:
        return not bool(self._config_dict)


class BaseToolConfig:
    def __init__(self, config_dict: Dict = None) -> None:
        self._config = Config(config_dict)

    def has_config(self) -> bool:
        return not self._config.is_empty()

    def get_config(self) -> Config:
        return self._config


class AutohooksConfig(BaseToolConfig):
    def __init__(self, config_dict: Dict = None) -> None:
        super().__init__(config_dict)
        self._autohooks_config = self._config.get("tool").get("autohooks")

    def has_autohooks_config(self) -> bool:
        return not self._autohooks_config.is_empty()

    def is_autohooks_enabled(self) -> bool:
        return self.has_autohooks_config()

    def get_pre_commit_script_names(self) -> List[str]:
        if self.has_autohooks_config():
            return self._autohooks_config.get_value("pre-commit", [])

        return []

    def get_mode(self) -> Mode:
        if self.has_autohooks_config():
            mode = self._autohooks_config.get_value("mode")
            if not mode:
                return Mode.UNDEFINED

            mode = Mode.from_string(mode.upper())
            is_virtual_env = mode == Mode.PIPENV or mode == Mode.POETRY
            if is_virtual_env and not is_split_env():
                if mode == Mode.POETRY:
                    mode = Mode.POETRY_MULTILINE
                else:
                    mode = Mode.PIPENV_MULTILINE
            return mode

        return Mode.UNDEFINED

    @staticmethod
    def from_pyproject_toml(pyproject_toml: Path = None) -> "AutohooksConfig":
        if pyproject_toml is None:
            pyproject_toml = get_pyproject_toml_path()

        if not pyproject_toml.exists():
            return AutohooksConfig()

        config_dict = tomlkit.loads(pyproject_toml.read_text())
        return AutohooksConfig(config_dict)


def load_config_from_pyproject_toml(
    pyproject_toml: Path = None,
) -> AutohooksConfig:
    return AutohooksConfig.from_pyproject_toml(pyproject_toml)



# Copyright (C) 2019-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path

from autohooks.settings import Mode
from autohooks.template import (
    PIPENV_MULTILINE_SHEBANG,
    PIPENV_SHEBANG,
    POETRY_MULTILINE_SHEBANG,
    POETRY_SHEBANG,
    PYTHON3_SHEBANG,
    TEMPLATE_VERSION,
    PreCommitTemplate,
)
from autohooks.utils import get_git_hook_directory_path


def get_pre_commit_hook_path():
    git_hook_dir_path = get_git_hook_directory_path()
    return git_hook_dir_path / "pre-commit"


class PreCommitHook:
    def __init__(self, pre_commit_hook_path: Path = None) -> None:
        self._pre_commit_hook = None

        if pre_commit_hook_path is None:
            self.pre_commit_hook_path = get_pre_commit_hook_path()
        else:
            self.pre_commit_hook_path = pre_commit_hook_path

    @property
    def pre_commit_hook(self) -> str:
        if self._pre_commit_hook is None:
            self._pre_commit_hook = self.pre_commit_hook_path.read_text()

        return self._pre_commit_hook

    def exists(self) -> bool:
        return self.pre_commit_hook_path.exists()

    def is_autohooks_pre_commit_hook(self) -> bool:
        lines = self.pre_commit_hook.split("\n")
        # seems to be false-positive ...
        return len(lines) > 5 and "autohooks.precommit" in self.pre_commit_hook

    def is_current_autohooks_pre_commit_hook(self) -> bool:
        return self.read_version() == TEMPLATE_VERSION

    def read_mode(self) -> Mode:
        lines = self.pre_commit_hook.split("\n")
        if len(lines) < 1 or len(lines[0]) == 0:
            return Mode.UNDEFINED

        shebang = lines[0][2:]

        if shebang == PYTHON3_SHEBANG:
            return Mode.PYTHONPATH
        if shebang == POETRY_SHEBANG:
            return Mode.POETRY
        if shebang == PIPENV_SHEBANG:
            return Mode.PIPENV

        shebang = f"{lines[0][2:]}\n"
        shebang += "\n".join(lines[1:5])
        if shebang == POETRY_MULTILINE_SHEBANG:
            return Mode.POETRY_MULTILINE

        if shebang == PIPENV_MULTILINE_SHEBANG:
            return Mode.PIPENV_MULTILINE

        return Mode.UNKNOWN

    def read_version(self) -> int:
        matches = re.search(
            r"{\s*version\s*=\s*?(\d+)\s*}$", self.pre_commit_hook, re.MULTILINE
        )
        if not matches:
            return -1

        return int(matches.group(1))

    def write(self, *, mode: Mode) -> None:
        template = PreCommitTemplate()
        pre_commit_hook = template.render(mode=mode)

        self.pre_commit_hook_path.write_text(pre_commit_hook)
        self.pre_commit_hook_path.chmod(0o775)

        self._pre_commit_hook = None

    def __str__(self) -> str:
        return str(self.pre_commit_hook_path)



# Copyright (C) 2019-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse

from autohooks.__version__ import __version__ as version
from autohooks.cli.activate import install_hooks
from autohooks.cli.check import check_hooks
from autohooks.settings import Mode
from autohooks.terminal import Terminal

DESCRIPTION = "autohooks - Manage git hooks"


def main():
    parser = argparse.ArgumentParser(description=DESCRIPTION)
    parser.add_argument(
        "--version",
        action="version",
        version=f"%(prog)s {version}",
    )

    subparsers = parser.add_subparsers(dest="command")
    activate_parser = subparsers.add_parser(
        "activate", help="Activate the pre-commit hook."
    )
    activate_parser.add_argument(
        "-f",
        "--force",
        action="store_true",
        help="Force activation of hook even if a hook already exists",
    )
    activate_parser.add_argument(
        "-m",
        "--mode",
        dest="mode",
        choices=[
            str(Mode.PYTHONPATH),
            str(Mode.PIPENV),
            str(Mode.POETRY),
        ],
        help="Mode for loading autohooks during hook execution. Either load "
        "autohooks from the PYTHON_PATH, via pipenv or via poetry.",
    )

    subparsers.add_parser("check", help="Check installed pre-commit hook")

    args = parser.parse_args()

    if not args.command:
        parser.print_usage()

    term = Terminal()
    if args.command == "activate":
        install_hooks(term, args)
    elif args.command == "check":
        check_hooks(term)


if __name__ == "__main__":
    main()



# Copyright (C) 2019-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path
from string import Template

from autohooks.settings import Mode
from autohooks.utils import get_autohooks_directory_path

PYTHON3_SHEBANG = "/usr/bin/env python3"
PIPENV_SHEBANG = "/usr/bin/env -S pipenv run python3"
POETRY_SHEBANG = "/usr/bin/env -S poetry run python3"
# For OS's that don't support '/usr/bin/env -S'.
PIPENV_MULTILINE_SHEBANG = (
    "/bin/sh\n"
    "\"true\" ''':'\n"
    'pipenv run python3 "$0" "$@"\n'
    'exit "$?"\n'
    "'''"
)
POETRY_MULTILINE_SHEBANG = (
    "/bin/sh\n"
    "\"true\" ''':'\n"
    'poetry run python3 "$0" "$@"\n'
    'exit "$?"\n'
    "'''"
)

TEMPLATE_VERSION = 1


def get_pre_commit_hook_template_path() -> Path:
    setup_dir_path = get_autohooks_directory_path()
    return setup_dir_path / "precommit" / "template"


class PreCommitTemplate:
    def __init__(self, template_path: Path = None) -> None:
        if template_path is None:
            template_path = get_pre_commit_hook_template_path()
        self._load(template_path)

    def _load(self, template_path: Path) -> None:
        self._template = Template(template_path.read_text())

    def render(self, *, mode: Mode) -> str:
        mode = mode.get_effective_mode()

        params = dict(VERSION=TEMPLATE_VERSION)

        if mode == Mode.PIPENV:
            params["SHEBANG"] = PIPENV_SHEBANG
        elif mode == Mode.POETRY:
            params["SHEBANG"] = POETRY_SHEBANG
        elif mode == Mode.PIPENV_MULTILINE:
            params["SHEBANG"] = PIPENV_MULTILINE_SHEBANG
        elif mode == Mode.POETRY_MULTILINE:
            params["SHEBANG"] = POETRY_MULTILINE_SHEBANG
        else:
            params["SHEBANG"] = PYTHON3_SHEBANG

        return self._template.safe_substitute(params)



# Copyright (C) 2019-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import fnmatch
import os
from os import PathLike
from pathlib import Path
from typing import Iterable


def is_python_path(path: Path) -> bool:
    if not path:
        return False
    return path.match("*.py")


def match(path: PathLike, pattern_list: Iterable) -> bool:
    """
    Check if a Path matches to one of the patterns

    Internally fnmatch is used.
    See https://docs.python.org/3/library/fnmatch.html for details.

    Arguments:
        path (Path): Path to check if it matches to one of the patterns
        pattern_list (iterable): Iterable (e.g tuple or list) of patterns to
            match against the path

    Returns:
        Boolean: True if path matches a pattern of the list
    """
    for pattern in pattern_list:
        if fnmatch.fnmatch(os.fspath(path), pattern):
            return True
    return False



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['autohooks',
 'autohooks.api',
 'autohooks.cli',
 'autohooks.precommit',
 'tests',
 'tests.api',
 'tests.api.git',
 'tests.cli']

package_data = \
{'': ['*']}

modules = \
['CHANGELOG', 'RELEASE', 'poetry']
install_requires = \
['pontos>=22.8.0', 'rich>=12.5.1,<13.0.0', 'tomlkit>=0.5.11']

entry_points = \
{'console_scripts': ['autohooks = autohooks.cli:main']}

setup_kwargs = {
    'name': 'autohooks',
    'version': '22.8.0',
    'description': 'Library for managing git hooks',
    'long_description': '![Greenbone Logo](https://www.greenbone.net/wp-content/uploads/gb_new-logo_horizontal_rgb_small.png)\n# Autohooks <!-- omit in toc -->\n\n[![PyPI release](https://img.shields.io/pypi/v/autohooks.svg)](https://pypi.org/project/autohooks/)\n[![Build and test Python package](https://github.com/greenbone/autohooks/actions/workflows/ci-python.yml/badge.svg)](https://github.com/greenbone/autohooks/actions/workflows/ci-python.yml)\n[![codecov](https://codecov.io/gh/greenbone/autohooks/branch/main/graph/badge.svg?token=9IX7ucaFwj)](https://codecov.io/gh/greenbone/autohooks)\n\nLibrary for managing and writing [git hooks](https://git-scm.com/docs/githooks)\nin Python.\n\nLooking for automatic formatting or linting, e.g., with [black] and [pylint],\nwhile creating a git commit using a pure Python implementation?\nWelcome to **autohooks**!\n\n- [Why?](#why)\n- [Solution](#solution)\n- [Requirements](#requirements)\n- [Modes](#modes)\n  - [Pythonpath Mode](#pythonpath-mode)\n  - [Poetry Mode](#poetry-mode)\n  - [Pipenv Mode](#pipenv-mode)\n- [Installing autohooks](#installing-autohooks)\n  - [1. Choosing an autohooks Mode](#1-choosing-an-autohooks-mode)\n  - [2. Installing the autohooks Python Package into the Current Environment](#2-installing-the-autohooks-python-package-into-the-current-environment)\n  - [3. Configuring Plugins to Be Run](#3-configuring-plugins-to-be-run)\n  - [4. Activating the Git Hooks](#4-activating-the-git-hooks)\n- [Plugins](#plugins)\n- [Howto: Writing a Plugin](#howto-writing-a-plugin)\n  - [Linting Plugin](#linting-plugin)\n  - [Formatting Plugin](#formatting-plugin)\n- [Maintainer](#maintainer)\n- [Contributing](#contributing)\n- [License](#license)\n\n## Why?\n\nSeveral outstanding libraries for managing and executing git hooks exist already.\nTo name a few: [husky](https://github.com/typicode/husky),\n[lint-staged](https://github.com/okonet/lint-staged),\n[precise-commits](https://github.com/nrwl/precise-commits) or\n[pre-commit](https://github.com/pre-commit/pre-commit).\n\nHowever, they either need another interpreter besides python (like husky) or are\ntoo ambiguous (like pre-commit). pre-commit is written in python but has support\nhooks written in all kind of languages. Additionally, it maintains the dependencies by\nitself and does not install them in the current environment.\n\n## Solution\n\nautohooks is a pure python library that installs a minimal\n[executable git hook](https://github.com/greenbone/autohooks/blob/main/autohooks/precommit/template).\nIt allows the decision of how to maintain the hook dependencies\nby supporting different modes.\n\n![Autohooks](https://raw.githubusercontent.com/greenbone/autohooks/main/autohooks.gif)\n\n## Requirements\n\nPython 3.7+ is required for autohooks.\n\n## Modes\n\nCurrently three modes for using autohooks are supported:\n\n* `pythonpath`\n* `poetry`\n* `pipenv`\n\nThese modes handle how autohooks, the plugins and their dependencies are loaded\nduring git hook execution.\n\nIf no mode is specified in the [`pyproject.toml` config file](#configure-mode-and-plugins-to-be-run)\nand no mode is set during [activation](#activating-the-git-hooks), autohooks\nwill use the [pythonpath mode](#pythonpath-mode) by default.\n\n`poetry` or `pipenv` modes leverage the `/usr/bin/env` command using the\n`--split-string` (`-S`) option. If `autohooks` detects that it is\nrunning on an OS where `/usr/bin/env` is yet to support _split_strings_\n(notably ubuntu < 19.x), `autohooks` will automatically change to an\ninternally chosen `poetry_multiline`/`pipenv_mutliline` mode. The\n\'multiline\' modes *should not* be user-configured options; setting your\nproject to use `poetry` or `pipenv`allows team members the greatest\nlatitude to use an OS of their choice yet leverage the sane\n`/usr/bin/env --split-string` if possible. Though `poetry_multiline`\nwould generally work for all, it is very confusing sorcery.\n([Multiline shebang explained](https://rosettacode.org/wiki/Multiline_shebang#Python))\n\n### Pythonpath Mode\n\nIn the `pythonpath` mode, the user has to install autohooks, the desired\nplugins and their dependencies into the [PYTHONPATH](https://docs.python.org/3/library/sys.html#sys.path)\nmanually.\n\nThis can be achieved by running `python3 -m pip install --user autohooks ...` to put them\ninto the installation directory of the [current user](https://docs.python.org/3/library/site.html#site.USER_SITE)\nor with `python3 -m pip install autohooks ...` for a system wide installation.\n\nAlternatively, a [virtual environment](https://packaging.python.org/tutorials/installing-packages/#creating-and-using-virtual-environments)\ncould be used separating the installation from the global and user wide\nPython packages.\n\nIt is also possible to use [pipenv] for managing the virtual\nenvironment but activating the environment has to be done manually.\n\nTherefore it is even possible to run different versions of autohooks by\nusing the `pythonpath` mode and switching to a virtual environment.\n\n### Poetry Mode\n\nLike with the [pipenv mode](#pipenv-mode), it is possible to run autohooks in a\ndedicated environment controlled by [poetry]. By using the `poetry` mode the\nvirtual environment will be activated automatically in the background when\nexecuting the autohooks based git commit hook.\n\nUsing the `poetry` mode is highly recommended.\n\n### Pipenv Mode\n\nIn the `pipenv` mode [pipenv] is used to run autohooks in a dedicated virtual\nenvironment. Pipenv uses a lock file to install exact versions. Therefore the\ninstallation is deterministic and reliable between different developer setups.\nIn contrast to the `pythonpath` mode the activation of the virtual environment\nprovided by [pipenv] is done automatically in the background.\n\n## Installing autohooks\n\nFour steps are necessary for installing autohooks:\n\n1. Choosing an autohooks mode\n2. Installing the autohooks python package into the current environment\n3. Configuring plugins to be run\n4. Activating the [git hooks](https://git-scm.com/docs/githooks)\n\n### 1. Choosing an autohooks Mode\n\nFor its configuration, autohooks uses the *pyproject.toml* file specified in\n[PEP518](https://www.python.org/dev/peps/pep-0518/).\nAdding a *[tool.autohooks]* section allows to specify the desired [autohooks mode](#modes)\nand to set python modules to be run as [autohooks plugins](#plugins).\n\nThe mode can be set by adding a `mode =` line to the *pyproject.toml* file.\nCurrent possible options are `"pythonpath"`, `"pipenv"` and `"poetry"` (see\n[autohooks mode](#modes)). If the mode setting is missing, the `pythonpath` mode is used.\n\nExample *pyproject.toml*:\n\n```toml\n[tool.autohooks]\nmode = "pipenv"\n```\n\n### 2. Installing the autohooks Python Package into the Current Environment\n\nUsing [poetry] is highly recommended for installing the autohooks python package.\n\nTo install autohooks as a development dependency run\n\n```sh\npoetry add --dev autohooks\n```\n\nAlternatively, autohooks can be installed directly from GitHub by running\n\n```sh\npoetry add --dev git+https://github.com/greenbone/autohooks\n```\n\n### 3. Configuring Plugins to Be Run\n\nTo actually run an action on git hooks, [autohooks plugins](#plugins) have to be\ninstalled and configured, e.g., to install python linting via pylint run\n\n```bash\npoetry add --dev autohooks-plugin-pylint\n```\n\nAfterwards, the pylint plugin can be configured to run as a pre-commit git hook\nby adding the autohooks-plugins-pylint python module name to the `pre-commit`\nsetting in the `[tool.autohooks]` section in the *pyproject.toml* file.\n\nExample *pyproject.toml*:\n\n```toml\n[tool.autohooks]\nmode = "pipenv"\npre-commit = ["autohooks.plugins.pylint"]\n```\n\n### 4. Activating the Git Hooks\n\nBecause installing and activating git hooks automatically isn\'t reliable (with\nusing source distributions and different versions of pip) and even impossible\n(with using [wheels](https://www.python.org/dev/peps/pep-0427/)) the hooks need\nto be activated manually once in each installation.\n\nTo activate the git hooks run\n\n```bash\npoetry run autohooks activate\n```\n\nCalling `activate` also allows for overriding the [mode](#modes) defined in the\n*pyproject.toml* settings for testing purposes.\n\nExample:\n\n\n```bash\nautohooks activate --mode pipenv\n```\n\nPlease keep in mind that autohooks will always issue a warning if the mode used\nin the git hooks is different from the configured mode in the *pyproject.toml*\nfile.\n\nThe activation can always be verified by running `autohooks check`.\n\n## Plugins\n\n* Python code formatting via [black](https://github.com/greenbone/autohooks-plugin-black)\n\n* Python code formatting via [autopep8](https://github.com/LeoIV/autohooks-plugin-autopep8)\n\n* Python code linting via [pylint](https://github.com/greenbone/autohooks-plugin-pylint)\n\n* Python code linting via [flake8](https://github.com/greenbone/autohooks-plugin-flake8)\n\n* Python import sorting via [isort](https://github.com/greenbone/autohooks-plugin-isort)\n\n* Running tests via [pytest](https://github.com/greenbone/autohooks-plugin-pytest/)\n\n## Howto: Writing a Plugin\n\nPlugins need to be available in the\n[Python import path](https://docs.python.org/3/reference/import.html). The\neasiest way to achieve this is uploading a plugin to [PyPI](https://pypi.org/)\nand installing it via [pip] or [poetry].\n\nAlternatively, a plugin can also be put into a *.autohooks* directory in the root\ndirectory of the git repository where the hooks should be executed.\n\nAn autohooks plugin is a Python module which provides a **precommit** function.\nThe function must accept arbitrary keywords because the keywords are likely to\nchange in future. Therefore using **\\*\\*kwargs** is highly recommended.\nCurrently *config* and *report_progress* keyword arguments are passed to the\nprecommit function.\n\nExample:\n\n```python3\ndef precommit(config=None, report_progress=None, **kwargs):\n```\n\nThe config can be used to receive settings from the *pyproject.toml* file, e.g.,\n\n```toml\n[tool.autohooks.plugins.foo]\nbar = 2\n```\n\ncan be received with\n\n```python3\ndef precommit(**kwargs):\n    config = kwargs.get(\'config\')\n    default_value = 1\n    setting = config\n      .get(\'tool\', \'autohooks\', \'plugins\', \'foo\')\n      .get_value(\'bar\', default_value)\n    return 0\n```\n\nThe report_progress can be used since autohooks 22.8.0 to display a progress bar\nwhen running a plugin.\n\n```python3\ndef precommit(report_progress, **kwargs):\n    report_progress.init(len(files))\n\n    for file in files:\n      check_file(file)\n      report_progress.update()\n\n    return 0\n```\nWith autohooks it is possible to write all kinds of plugins. Most common are\nplugins for linting and formatting.\n\n### Linting Plugin\n\nUsually the standard call sequence for a linting plugin is the following:\n\n1. get list of staged files\n2. filter list of files for a specific file type\n3. stash unrelated changes\n4. apply checks on filtered list of files by calling some external tool\n5. raise exception if something did go wrong\n6. return 1 if check was not successful\n6. stage changes made by the tool\n7. unstash unrelated changes\n8. return 0\n\nExample plugin:\n\n```python3\nimport subprocess\n\nfrom autohooks.api import ok, fail\nfrom autohooks.api.git import get_staged_status, stash_unstaged_changes\nfrom autohooks.api.path import match\n\nDEFAULT_INCLUDE = (\'*.ext\')\n\n\ndef get_include(config)\n    if not config:\n        return DEFAULT_INCLUDE\n\n    config = config.get(\'tool\', \'autohooks\', \'plugins\', \'foo\')\n    return config.get_value(\'include\', DEFAULT_INCLUDE)\n\n\ndef precommit(**kwargs):\n    config = kwargs.get(\'config\')\n    include = get_include(config)\n\n    files = [f for f in get_staged_status() if match(f.path, include)]\n\n    if not files:\n      # not files to lint\n      return 0\n\n    with stash_unstaged_changes(files):\n        const failed = False\n        for file in files:\n            status = subprocess.call([\'foolinter\', str(file)])\n            if status:\n                fail(\'Could not validate {str(file)}\')\n                failed = True\n            else:\n                ok(\'Validated {str(file)}\')\n\n        return 1 if failed else 0\n```\n\n### Formatting Plugin\n\nUsually the standard call sequence for a formatting plugin is the following:\n\n1. get list of staged files\n2. filter list of files for a specific file type\n3. stash unrelated changes\n4. apply formatting on filtered list of files by calling some external tool\n5. raise exception if something did go wrong\n6. stage changes made by the tool\n7. unstash unrelated changes\n8. return 0\n\nExample plugin:\n\n```python3\nimport subprocess\n\nfrom autohooks.api import ok, error\nfrom autohooks.api.git import (\n    get_staged_status,\n    stage_files_from_status_list,\n    stash_unstaged_changes,\n)\nfrom autohooks.api.path import match\n\nDEFAULT_INCLUDE = (\'*.ext\')\n\n\ndef get_include(config)\n    if not config:\n        return DEFAULT_INCLUDE\n\n    config = config.get(\'tool\', \'autohooks\', \'plugins\', \'bar\')\n    return config.get_value(\'include\', DEFAULT_INCLUDE)\n\n\ndef precommit(**kwargs):\n    config = kwargs.get(\'config\')\n    include = get_include(config)\n\n    files = [f for f in get_staged_status() if match(f.path, include)]\n\n    if not files:\n      # not files to format\n      return 0\n\n    with stash_unstaged_changes(files):\n        for file in files:\n            # run formatter and raise exception if it fails\n            subprocess.run([\'barformatter\', str(file)], check=True)\n            ok(\'Formatted {str(file)}\')\n\n        return 0\n```\n\n## Maintainer\n\nThis project is maintained by [Greenbone Networks GmbH](https://www.greenbone.net/).\n\n## Contributing\n\nYour contributions are highly appreciated. Please\n[create a pull request](https://github.com/greenbone/autohooks/pulls)\non GitHub. Bigger changes need to be discussed with the development team via the\n[issues section at GitHub](https://github.com/greenbone/autohooks/issues)\nfirst.\n\n## License\n\nCopyright (C) 2019 - 2022 [Greenbone Networks GmbH](https://www.greenbone.net/)\n\nLicensed under the [GNU General Public License v3.0 or later](LICENSE).\n\n[black]: https://black.readthedocs.io/en/stable/\n[pip]: https://pip.pypa.io/en/stable/\n[pipenv]: https://pipenv.readthedocs.io/en/latest/\n[poetry]: https://python-poetry.org/\n[pylint]: https://pylint.readthedocs.io/en/latest/\n',
    'author': 'Greenbone Networks GmbH',
    'author_email': 'info@greenbone.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/greenbone/autohooks',
    'packages': packages,
    'package_data': package_data,
    'py_modules': modules,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import random
import tempfile
import unittest
from contextlib import contextmanager
from pathlib import Path
from typing import Generator

from autohooks.utils import exec_git


def randbytes(n: int) -> bytes:  # pylint: disable=invalid-name
    if hasattr(random, "randbytes"):
        return random.randbytes(n)
    return random.getrandbits(n * 8).to_bytes(n, "little")


def git_add(*paths: Path) -> None:
    exec_git("add", *paths)


def git_rm(*paths: Path) -> None:
    exec_git("rm", *paths)


def git_mv(from_path: Path, to_path: Path) -> None:
    exec_git("mv", from_path, to_path)


def git_commit(message: str = "Foo Bar"):
    exec_git("commit", "--no-gpg-sign", "-m", message)


class GitTestCase(unittest.TestCase):
    pass



# Copyright (C) 2019-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from argparse import Namespace

from autohooks.config import (
    get_pyproject_toml_path,
    load_config_from_pyproject_toml,
)
from autohooks.hooks import PreCommitHook
from autohooks.settings import Mode
from autohooks.terminal import Terminal


def install_hooks(term: Terminal, args: Namespace) -> None:
    pre_commit_hook = PreCommitHook()
    pyproject_toml = get_pyproject_toml_path()
    config = load_config_from_pyproject_toml(pyproject_toml)

    if pre_commit_hook.exists() and not args.force:
        term.ok(
            "autohooks pre-commit hook is already"
            f" installed at {str(pre_commit_hook)}."
        )
        with term.indent():
            term.print()
            term.info(
                "Run 'autohooks activate --force' to override the current "
                "installed pre-commit hook."
            )
            term.info(
                "Run 'autohooks check' to validate the current status of "
                "the installed pre-commit hook."
            )
    else:
        if not config.is_autohooks_enabled():
            term.warning(
                f"autohooks is not enabled in your {str(pyproject_toml)} "
                "file. Run 'autohooks check' for more details."
            )

        if args.mode:
            mode = Mode.from_string(args.mode)
        else:
            mode = config.get_mode()

        pre_commit_hook.write(mode=mode)

        term.ok(
            f"autohooks pre-commit hook installed at {str(pre_commit_hook)}"
            f" using {str(mode.get_effective_mode())} mode."
        )



# Copyright (C) 2019-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
from unittest.mock import Mock

from autohooks.api import error, fail, info, ok, out, warning
from autohooks.terminal import Terminal, _set_terminal


class TerminalOutputApiTestCase(unittest.TestCase):
    def setUp(self):
        self.term = Mock(spec=Terminal)
        _set_terminal(self.term)

    def test_error(self):
        error("foo bar")
        self.term.error.assert_called_with("foo bar")

    def test_fail(self):
        fail("foo bar")
        self.term.fail.assert_called_with("foo bar")

    def test_info(self):
        info("foo bar")
        self.term.info.assert_called_with("foo bar")

    def test_ok(self):
        ok("foo bar")
        self.term.ok.assert_called_with("foo bar")

    def test_out(self):
        out("foo bar")
        self.term.print.assert_called_with("foo bar")

    def test_warning(self):
        warning("foo bar")
        self.term.warning.assert_called_with("foo bar")


if __name__ == "__main__":
    unittest.main()
